@@ -241,31 +241,37 @@ class Sectile(object):
         match['dimensions']['path'] = None
         return match
 
-    def get_dimension_possibilities(self, fragment, path, **kwargs):
+    def get_dimension_possibilities(self, path, **kwargs):
         dimension_possibilities = []
         for dimension in self.get_dimensions_list():
             possibilities = {'name': dimension}
-            if dimension in kwargs:
+            if dimension in kwargs and kwargs[dimension] != 'all':
                 possibilities['options'] \
                     = self.get_dimension_inheritance(
                         dimension, kwargs[dimension])
             else:
                 possibilities['options'] = ['all']
             dimension_possibilities.append(possibilities)
-        paths = {'name': 'path', 'options': []}
-        for subdir in self.split_path(path):
-            paths['options'].append(os.path.join(subdir, fragment))
-        dimension_possibilities.append(paths)
+        dimension_possibilities.append({
+                'name': 'path',
+                'options': self.split_path(path)
+            })
 
         return dimension_possibilities
 
-    def get_fragment_paths(self, fragment, path, **kwargs):
+    def get_dimension_paths(self, path, **kwargs):
         possibilities \
-            = self.get_dimension_possibilities(fragment, path, **kwargs)
+            = self.get_dimension_possibilities(path, **kwargs)
 
-        # move the path (last possibility returned) to the start
-        # so that it sorts correctly when coming out of itertools.product
-        possibilities.insert(0, possibilities.pop())
+        # move the path to the start of the possibilities, so that everything
+        # sorts correctly when coming out of itertools.product
+        # (adding a blank to the paths so they end with '/')
+        path_poss = possibilities.pop()
+        path_poss['options'] = [
+            os.path.join(x, '')
+                for x in path_poss['options']
+        ]
+        possibilities.insert(0, path_poss)
 
         paths = []
         options = [ poss['options'] for poss in possibilities ]
@@ -282,6 +288,12 @@ class Sectile(object):
             paths.append(os.path.join('default', path))
         return paths
 
+    def get_fragment_paths(self, fragment, path, **kwargs):
+        return [
+            os.path.join(possibility, fragment)
+                for possibility in self.get_dimension_paths(path, **kwargs)
+            ]
+
     def split_path(self, path):
         if path.find('/') >= 0:
             path = os.path.normpath(path).lstrip('/')
@@ -298,6 +310,55 @@ class Sectile(object):
         with open(path) as handle:
             return handle.read()
 
+    def create_fragment(self, fragment, path, **kwargs):
+        paths = []
+        for dimension in self.get_dimensions_list():
+            if dimension in kwargs:
+                paths.append(kwargs[dimension])
+            else:
+                paths.append('all')
+        # special case, if every dimension is 'all' then we use 'default'
+        # ie. all/all/all/thing == default/thing
+        if paths.count('all') == len(paths):
+            paths=['default']
+        paths.append(path)
+
+        target_fragment = os.path.join(*paths, fragment)
+
+        # find the less specific option, and copy it
+        match = self.get_matching_fragment(fragment, path, **kwargs)
+        if match['found']:
+            if match['found'] == target_fragment:
+                raise FileExistsError
+            content = self.get_fragment_file(match['found'])
+        else:
+            content = ''
+
+        target_dir = os.path.join(
+            self.fragments_directory,
+            *paths,
+        )
+
+        if not os.path.exists(target_dir):
+            os.makedirs(target_dir)
+        with open(os.path.join(target_dir, fragment), 'w') as handle:
+            handle.write(content)
+
+    def update_fragment(self, fragment, content):
+        file = os.path.join(
+                self.fragments_directory,
+                fragment,
+            )
+        with open(file, 'w') as handle:
+            handle.write(content)
+
+    def delete_fragment(self, fragment):
+        file = os.path.join(
+                self.fragments_directory,
+                fragment,
+            )
+        os.remove(file)
+
     def build_dimensions(self, dimensions):
         if not dimensions:
             try:
